#ifndef SIMPLEDCE_H_
#define SIMPLEDCE_H_

#include "llvm/CodeGen/LiveVariables.h"
#include "llvm/Function.h"
#include "llvm/CodeGen/MachineFunctionPass.h"
#include "llvm/CodeGen/MachineInstr.h"
#include "llvm/CodeGen/Passes.h"
#include "llvm/Target/TargetInstrInfo.h"
#include "llvm/Target/TargetMachine.h"
#include "llvm/ADT/Statistic.h"
#include "llvm/ADT/STLExtras.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/Compiler.h"

/* Does one iteration of dead code elimination.  Since LiveVariables doesn't
 * update properly, this is all we can do. */

using namespace llvm;

namespace llvm
{
class SimpleDCE : public MachineFunctionPass
{
public:
	static char ID;
	SimpleDCE() :
		MachineFunctionPass((intptr_t)&ID)
	{
	}

	virtual ~SimpleDCE() {}

	virtual const char* getPassName() const
	{
		return "Simple Dead Code Elimination";
	}

	virtual void getAnalysisUsage(AnalysisUsage &AU) const
	{		
		AU.addRequired<LiveVariables>();
		AU.addPreservedID(PHIEliminationID);
		AU.addPreservedID(TwoAddressInstructionPassID);
		MachineFunctionPass::getAnalysisUsage(AU);
	}

	//remove dead code from function, assumes liveness is up to date
	static int removeDeadCode(MachineFunction&);
	
	/// runOnMachineFunction -remove dead code
	bool runOnMachineFunction(MachineFunction&);
};

}
#endif /*SIMPLEDCE_H_*/
