#ifndef ALLOCCLASSMAP_H_
#define ALLOCCLASSMAP_H_

#include "llvm/PrincipledBackend/ProgCommon.h"
#include "llvm/PrincipledBackend/GlobalMCNF.h"
#include "llvm/PrincipledBackend/AllocationClassInfo.h"
#include "llvm/ADT/hash_map"

namespace llvm {


class AllocClassMap
{
	MachineFunction* MF;
	const TargetMachine* TM;
	const MRegisterInfo* RegInfo;

	mcnf_block *block;

	hash_map<int, unsigned> regMap; //quantityto physical reg
	hash_set<int> stackMap; //quantity is in memory
	hash_map<unsigned, int> reg_quantity; //map from llvm reg to quantity
	hash_map<int, unsigned> quantity_regs; //map from quantity to current llvm reg

public:
	AllocClassMap(mcnf_block *b);
	virtual ~AllocClassMap() {}

	//set quantity q to be mapped to allocClass
	void set_quantity_alloc_class(int q, int allocClass, unsigned r);
	//remove the mapping
	void unset_quantity_alloc_class(int q);
	//sets the current reg being used by the quantity
	void set_quantity_reg(int q, unsigned r);

	//return true if llvm register r is mapped to a physical register
	bool isReg(unsigned r) { return regMap.count(r) > 0; }
	//return true if r is currently in mem
	bool isMem(unsigned r) { return stackMap.count(r) > 0; }
	//return reg number for llvm register r
	//if r is allocated to a phys reg, is the phys reg number,
	//otherwise is the register itself
	unsigned getMappedReg(unsigned r);
	//return reg number corresponding to a quantity
	unsigned getMappedQuantity(int q);
	//looks up the current quantity reg
	unsigned getRegForQuantity(int q);

};

} //namespace llvm
#endif /*ALLOCCLASSMAP_H_*/
