#ifndef BITSET_H_
#define BITSET_H_

#include <vector>
#include "llvm/ADT/hash_set"
#include "llvm/ADT/UnReservedVector.h"

namespace llvm {

// Automatically resizing bitset
class BitSet
{
	UnReservedVector<unsigned char> bits;
	
	//convert pos/neg value to unsigned index
	unsigned index(int b) const;
	
public:	
	BitSet() {}
	BitSet(const BitSet& RHS): bits(RHS.bits) {}
	bool test(int b) const //inline
	{
		unsigned i = index(b);
		unsigned pos = i/8;
		unsigned bpos = i%8;
		if(pos >= bits.size())
			return false;
		
		unsigned char byt = bits[pos];
		return byt & (1<<bpos);
	}	void set(int b);	
	bool operator==(const BitSet& rhs) const;	
	
	void reserve(unsigned sz); //reserve space for sz bits
	friend struct BitSetHasher;
};
   
struct BitSetHasher
{
	size_t operator()(const BitSet& set) const;
};

/* Keep only a single copy of any unique bitset */
class MemoizedBitSet
{
	typedef hash_set<BitSet, BitSetHasher> BitHash;
	static BitHash sets;
	const BitSet *myset;
	
	//return the unique pointer to this set, inserting if necessary
	const BitSet* getPtr(const BitSet& set);
	
public:
	
	MemoizedBitSet();
	bool test(int b);	
	void set(int b);
	
	static unsigned numUnique() { return sets.size(); }
};

}
#endif /*BITSET_H_*/
