#include "llvm/PrincipledBackend/AllocClassMap.h"
#include "llvm/PrincipledBackend/ProgCommon.h"

namespace llvm
{

AllocClassMap::AllocClassMap(mcnf_block *b) :
	block(b)
{
	MF = block->b->getParent();
	TM = &MF->getTarget();
	RegInfo = TM->getRegisterInfo();
}

//set quantity q to be mapped to allocClass
void AllocClassMap::set_quantity_alloc_class(int q, int allocClass, unsigned r)
{
	if(alloc_classes->getType(allocClass) == AllocationClassInfo::Register)
	{
		MF->setPhysRegUsed(r); //necessary for fp stackifier to run
		regMap[q] = r;
		stackMap.erase(q);
	}
	else
	{
		//doesn't get a physical register
		stackMap.insert(q);
		regMap.erase(q);
	}
}

void AllocClassMap::unset_quantity_alloc_class(int q)
{
	assert(quantity_regs.count(q) > 0);
	unsigned r = quantity_regs[q];
	regMap.erase(q);
	stackMap.erase(q);
	quantity_regs.erase(q);
	reg_quantity.erase(r);
}

//sets the current reg being used by the quantity
void AllocClassMap::set_quantity_reg(int q, unsigned r)
{
	if(quantity_regs.count(q) > 0)
	{
		unsigned oldr = quantity_regs[q];
		reg_quantity[oldr] = 0;
	}
	reg_quantity[r] = q;
	quantity_regs[q] = r;
}

//return phys reg number for llvm register r
unsigned AllocClassMap::getMappedReg( unsigned r)
{
	assert(reg_quantity.count(r) > 0);
	int q = reg_quantity[r];
	assert(quantity_regs[q] ==  r);
	assert(q != 0);

	if(isMem(q))
		return r;
	assert(regMap.count(q) > 0 && "Attempt to use unmapped quantity");

	//compute the correct physical register (may need to use subreg)
	unsigned physReg = regMap[q];
	if(physReg == 0)
		abort();
	const TargetRegisterClass *RC = MF->getSSARegMap()->getRegClass(r);
	if(!RC->allocableContains(*MF, physReg)) //need a subreg
	{
		const unsigned *subs = RegInfo->get(physReg).SubRegs;
		while(*subs)
		{
			unsigned sub = *subs;
			if(RC->allocableContains(*MF, sub))
			{
				physReg = sub;
				break;
			}
			subs++;
		}

		//if we couldn't find a subregister,
		//trust that the solution is correct and it's okay to switch
		//to an incompatable register
	}
	return physReg;
}

//return phys reg number for quantity q
unsigned AllocClassMap::getMappedQuantity(int q)
{
	assert(quantity_regs.count(q) > 0);
	unsigned r = quantity_regs[q];

	return getMappedReg(r);
}

//looks up the current quantity reg
unsigned AllocClassMap::getRegForQuantity(int q)
{
	assert(quantity_regs.count(q) > 0 && quantity_regs[q] > 0 && "Unmapped quantity");
	return quantity_regs[q];
}
}
