#include "llvm/PrincipledBackend/BitSet.h"
#include <assert.h>

namespace llvm
{

//convert pos/neg value to unsigned index
unsigned BitSet::index(int b) const
{
	int ret = b*2;
	if (ret<0)
	{
		ret = -ret;
		ret--;
	}
	return (unsigned)ret;
}



void BitSet::set(int b)
{
	unsigned i = index(b);
	unsigned pos = i/8;
	unsigned bpos = i%8;
	if(pos >= bits.size())
		bits.resize(pos+1, 0);
	bits[pos] |= (1<<bpos);
}

void BitSet::reserve(unsigned sz)
{
	unsigned numb = sz/8 + 1;
	if(bits.size() < numb)
		bits.resize(numb, 0);
}

bool BitSet::operator==(const BitSet& rhs) const
{
	return bits == rhs.bits;
}

MemoizedBitSet::BitHash MemoizedBitSet::sets;

//return the unique pointer to this set, inserting if necessary
const BitSet* MemoizedBitSet::getPtr(const BitSet& set)
{
	BitHash::const_iterator pos = sets.find(set);
	if (pos == sets.end()) // have to insert

	{
		sets.insert(set);
		pos = sets.find(set);
	}
	assert(pos != sets.end());
	return &*pos;
}

MemoizedBitSet::MemoizedBitSet()
{
	BitSet blank;
	myset = getPtr(blank);
}

bool MemoizedBitSet::test(int b)
{
	return myset->test(b);
}

void MemoizedBitSet::set(int b)
{
	if (!test(b))
	{
		BitSet newset = *myset;
		newset.set(b);
		myset = getPtr(newset);
	}
}

size_t BitSetHasher::operator()(const BitSet& set) const
{
	//sdbm hash function (from random website)
    size_t hash = 1; //make leading zeros significant
    
    for(unsigned i = 0, n = set.bits.size(); i < n; i++)
    {
    	unsigned char c = set.bits[i];
    	hash = c + (hash << 6) + (hash << 16) - hash;
    }
    return hash;	
}

}
