#ifndef LAGRANGIANSOLVER_H_
#define LAGRANGIANSOLVER_H_

#include "llvm/PrincipledBackend/GlobalMCNF.h"
#include "llvm/PrincipledBackend/Solution.h"
#include <iostream>
#include <stdio.h>
#include "llvm/Function.h"
#include "llvm/Module.h"
#include "llvm/ADT/StringExtras.h"
#include "llvm/Analysis/LoopInfo.h"
#include "BlockTraces.h"
#include "HeuristicSolver.h"
/*
 * Computes lagrangian prices iteratively to push heuristic solver towards
 * optimal solution.
 */
namespace llvm
{
class LagrangianSolver
{
	GlobalMCNF *mcnf_cfg;
	HeuristicSolver solver;
	Solution& solution;

	double mcnf_step;
	double norm;
	int current_iteration;
public:
	LagrangianSolver(GlobalMCNF *mcfg, LoopInfo *li, Solution& sol, bool pricesInitialized=false) :
		mcnf_cfg(mcfg), solver(mcfg, li), solution(sol)
		{
			norm = 1.0;
			if(!pricesInitialized)
				solver.initialize_dual_prices();
		}

	virtual ~LagrangianSolver()
	{

	}

	void changeModel(GlobalMCNF *newcfg) { mcnf_cfg = newcfg;  solver.changeModel(newcfg);}
	// return cost of best solution found
	int solve(int start, int end, bool pricesOnly = false);

private:

	/* Update the prices in the mcnf_cfg.
	   Assumes the old norm is in norm_ret and places the new norm into it.
	   w_new = [w + step * over/underflow]*[adjusted theta]
	   returns false if nothing is changed
	   */
	bool update_prices(double& norm_ret);

	/* Compute the norm of the gradient using the current
	   prices and flow information
	   If truncgrad is true, truncate gradients that
	   would put the price below zero
	   */
	double compute_grad_norm(int truncgrad);
	int max_cost_block_multiplier();
	bool dual_constraint_matters(mcnf_node *node, int c);

};
}

#endif /*LAGRANGIANSOLVER_H_*/
