#define DEBUG_TYPE "simpleDCE"

#include "llvm/CodeGen/SimpleDCE.h"
#include "llvm/Target/MRegisterInfo.h"

using namespace llvm;

namespace  llvm
{
STATISTIC(NumInsn, "Number of instructions removed");
char SimpleDCE::ID = 0;
static RegisterPass<SimpleDCE> X("simple-dce",
                     "Simple dead code elimination");

//remove dead code from function, assumes liveness is up to date
//is conservative about hard registers
//returns the amount (in bytes) of dead code removed
int SimpleDCE::removeDeadCode(MachineFunction& MF)
{
	const TargetMachine* TM = &MF.getTarget();
	const TargetInstrInfo *TII = TM->getInstrInfo();
	int ret = 0;
	for (MachineFunction::iterator MBB = MF.begin(), MBBe = MF.end(); MBB
				!= MBBe; ++MBB)
		{
		// loop over each instruction
		for (MachineBasicBlock::iterator MI = MBB->begin(); MI != MBB->end(); )
		{
			unsigned src, dst;
			if (TII->isMoveInstr(*MI, src, dst) && src == dst)
			{
					DOUT << "SimpleDCE: " << *MI;
					ret += TII->instrSize(*MI);
					MI = MBB->erase(MI);
					NumInsn++;
			}
			else if(TII->isMoveInstr(*MI, src, dst) && MI->getOperand(0).isDead()
					&& MRegisterInfo::isVirtualRegister(dst))
			{				
				DOUT << "SimpleDCE: " << *MI;
				ret += TII->instrSize(*MI);
				MI = MBB->erase(MI);
				NumInsn++;
			}
			//can eliminate extract subregs if both operands are the same reg,
			//but be safe and only do this if both are physical regs
			else if(MI->getOpcode() == TargetInstrInfo::EXTRACT_SUBREG && 
					MI->getOperand(0).isRegister() && MI->getOperand(1).isRegister() &&
					MRegisterInfo::isPhysicalRegister(MI->getOperand(0).getReg()))
			{
				unsigned SuperReg = MI->getOperand(1).getReg();
				unsigned SubIdx = MI->getOperand(2).getImm();
				unsigned SrcReg = TM->getRegisterInfo()->getSubReg(SuperReg, SubIdx);
				unsigned DstReg = MI->getOperand(0).getReg();

				if (SrcReg == DstReg)
				{
					DOUT << "SimpleDCE: " << *MI;
					ret += TII->instrSize(*MI);
					MI = MBB->erase(MI);
					NumInsn++;
				}
				else
					MI++;
			}
			else
				MI++;
		}
	}
	return ret;
}
	
// Remove simple dead code.  For now just remove obviously useless moves.
bool SimpleDCE::runOnMachineFunction(MachineFunction& MF)
{
	return removeDeadCode(MF);
}

}
